/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                       Copyright (C) A. P. H.                           **
**                           BIALYSTOK 2007                               **
**                                                                        **
**                         MODULE UART2 Service                           **
**                                                                        **
**            Mikrokontroler ARM: LPC2362..LPC2368, LPC2372..LPC2378      **
**                                                                        **
**                     Wersja z dnia : 2007.03.25                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#include <LPC23xx.h>
#include "uart2.h"
#include "types.h"
#include "system.h"
#include "irqvectors.h"
#include "serialconst.h"

#define UART2InterruptMask ( 1 << UART2_Interrupt )

#define InpSerialBuffSize     64
#define OutSerialBuffSize     128
                              /* 64 */

#define PCUART2               ( ULONG ) 1 << 24

typedef struct {
                   USHORT	    RdDataPointer ;
                   USHORT     WrDataPointer ;
                   UCHAR      Buffer [ InpSerialBuffSize ] ;
                 } InpSerialBufferRecT ;

typedef struct {
                   USHORT	    RdDataPointer ;
                   USHORT     WrDataPointer ;
                   USHORT     BusyStatus ;
                   UCHAR      Buffer [ OutSerialBuffSize ] ;
                 } OutSerialBufferRecT ;


static volatile InpSerialBufferRecT UART2InpBuffer ;
static volatile OutSerialBufferRecT UART2OutBuffer ;


__irq void UART2IrqHandler ( void )
{
  UCHAR Data ;
  UCHAR U2IrqStatus ;
  /*-------------------------------------------*/
  for ( ; ; )
  {
    U2IrqStatus = ( UCHAR ) U2IIR ;
    if ( U2IrqStatus & 0x01 )
    {
      VICVectAddr = 0 ;
      return ;
    } /* if */ ;
    switch ( ( U2IrqStatus >> 1 ) & 0x03 )
    {
      case 1             :   /* 0 1 - transmitter */
        if ( UART2OutBuffer . RdDataPointer == UART2OutBuffer . WrDataPointer )
        {
          UART2OutBuffer . BusyStatus = 0 ;
          break ;
        } /* if */ ;
        Data = UART2OutBuffer . Buffer [ UART2OutBuffer . RdDataPointer ] ;
        UART2OutBuffer . RdDataPointer ++ ;
        if ( UART2OutBuffer . RdDataPointer >= OutSerialBuffSize )
          UART2OutBuffer . RdDataPointer = 0 ;
        U2THR = Data ;
        break ;
      case 2             :   /* 1 0 - reciever data */
        Data = U2RBR ;
        UART2InpBuffer . Buffer [ UART2InpBuffer . WrDataPointer ] = Data ;
        UART2InpBuffer . WrDataPointer ++ ;
        if ( UART2InpBuffer . WrDataPointer >= InpSerialBuffSize )
          UART2InpBuffer . WrDataPointer = 0 ;
        break ;
      default            :
        VICVectAddr = 0 ;
        return ;
    } /* switch */ ;
  } /* for */ ;
} /* UART2IrqHandler */


void UART2InputReset ( void )
{
  UART2InpBuffer . RdDataPointer = 0 ;
  UART2InpBuffer . WrDataPointer = 0 ;
} /* UART2InputReset */


static void UART2OutputReset ( void )
{
  UART2OutBuffer . RdDataPointer = 0 ;
  UART2OutBuffer . WrDataPointer = 0 ;
  UART2OutBuffer . BusyStatus = 0 ;
} /* UART2OutputReset */


static void UART2DisableInterrupt ( void )
{
  VICIntEnClr = UART2InterruptMask ;
} /* UART2DisableInterrupt */


static void UART2EnableInterrupt ( void )
{
  VICIntEnable = UART2InterruptMask ;
} /* UART2EnableInterrupt */


void UART2Send ( UCHAR Data )
{
  USHORT WStop ;
  /*-------------------------------------------*/
  UART2DisableInterrupt ( ) ;
  if ( ! UART2OutBuffer . BusyStatus )
  {
    U2THR = Data ;
    UART2OutBuffer . BusyStatus = 1 ;
    UART2EnableInterrupt ( ) ;
  } /* if ... */
  else
  {
    for ( ; ; )
    {
      WStop = UART2OutBuffer . WrDataPointer + 1 ;
      if ( WStop >= OutSerialBuffSize )
        WStop = 0 ;
      if ( WStop != UART2OutBuffer . RdDataPointer )
      {
        UART2OutBuffer . Buffer [ UART2OutBuffer . WrDataPointer ] = Data ;
        UART2OutBuffer . WrDataPointer = WStop ;
        UART2EnableInterrupt ( ) ;
        break ;
      } /* if ... */
      UART2EnableInterrupt ( ) ;
      NOP ( ) ;
      NOP ( ) ;
      NOP ( ) ;
      NOP ( ) ;
      NOP ( ) ;
      UART2DisableInterrupt ( ) ;
    } /* for */ ;
  } /* if ... else */ ;
} /* UART2Send */


void UART2SendString ( UCHAR * String )
{
  while ( * String )
  {
    UART2Send ( * String ) ;
    String ++ ;
  } /* while */ ;
} /* UART2SendString */


ULONG UART2DataPresent ( void )
{
  ULONG Result ;
  /*-------------------------------------------*/
  Result = FALSE ;
  UART2DisableInterrupt ( ) ;
  if ( UART2InpBuffer . RdDataPointer != UART2InpBuffer . WrDataPointer )
  {
    Result = TRUE ;
  } /* if */ ;
  UART2EnableInterrupt ( ) ;
  return ( Result ) ;
} /* UART2DataPresent */


UCHAR UART2GetData ( void )
{
  UCHAR Data ;
  /*-------------------------------------------*/
  UART2DisableInterrupt ( ) ;
  if ( UART2InpBuffer . RdDataPointer == UART2InpBuffer . WrDataPointer )
  {
    UART2EnableInterrupt ( ) ;
    return ( 0 ) ;
  } /* if */ ;
  Data = UART2InpBuffer . Buffer [ UART2InpBuffer . RdDataPointer ] ;
  UART2InpBuffer . RdDataPointer ++ ;
  if ( UART2InpBuffer . RdDataPointer >= InpSerialBuffSize )
    UART2InpBuffer . RdDataPointer = 0 ;
  UART2EnableInterrupt ( ) ;
  return ( Data ) ;
} /* UARTGetData */


UCHAR UART2Working ( void )
{
  UCHAR Data ;
  /*------------------------*/
  if ( UART2OutBuffer . BusyStatus )
    return FALSE ;
  Data = U2LSR ;
  if ( Data  & 0x40 )
    return ( TRUE ) ;
  else
    return ( FALSE ) ;
} /* UART2Working */


void UART2HardwInit( ULONG PeriphClk ,
                     UCHAR SerialMode ,
                     ULONG SerialSpeed )
{
  ULONG Fdiv ;
  ULONG Fmod ;
  ULONG LDivider ;
  ULONG HDivider ;
  /*-------------------------------------------*/
  UART2DisableInterrupt ( ) ;
  UART2InputReset ( ) ;
  UART2OutputReset ( ) ;
  Fdiv = ( PeriphClk / 16 ) / SerialSpeed ;
  Fmod = ( PeriphClk / 16 ) % SerialSpeed ;
  if ( ( Fmod << 1 ) >= SerialSpeed )
    Fdiv ++ ;
  LDivider = Fdiv % 256 ;
  HDivider = Fdiv / 256 ;
  U2FDR = 0x10 ;
  U2LCR = 0x80 ;
  U2DLL = LDivider ;
  U2DLM = HDivider ;
  U2LCR = SerialMode ;
  U2FCR = 0x07 ;
  VICVectAddr28  = ( unsigned long ) UART2IrqHandler ;
  VICVectCntl28  = 15 ;
  UART2EnableInterrupt ( ) ;
  U2IER = RecvDataInterruptEnable | TransmDataInterruptEnable ;
} /* UART2HardwInit */


void UART2InitEnvir ( void )
{
  PCONP |= PCUART2 ;
  PINSEL4 &= ~ 0x000F0000  ;
  PINSEL4 |= 0x000A0000 ;  // P2.8 -> TXD2  P2.9 -> RXD2
  UART2InputReset ( ) ;
  UART2OutputReset ( ) ;
} /* UART2InitEnvir */

